class PushsController < ApplicationController
  include ActionController::HttpAuthentication::Token

  def token
    param! :app_name, String, required: true
    param! :app_id, String, required: true
    param! :app_secret, String, required: true

    app_setting = Settings.send(params[:app_name].to_sym) rescue {}
    if app_setting['app_id'] == params[:app_id] && app_setting['app_secret'] == params[:app_secret]
      render json: { code: 0, message: 'success', token: generate_token(params[:app_name]), ttl: token_ttl(params[:app_name])}
    else
      render json: { code: -1, message: '应用不存在或者app_id/app_secret不匹配'}
    end
  end

  def push_v2
    param! :app_name, String, required: true, in: %w(crm)
    # param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    # param! :igetui_opts, String
    param! :sync_push, :boolean
    param! :user_ids, Array, required: true
    param! :user_platform, String, required: true, in: UserDevice::user_platforms.keys

    message = JSON.parse(params[:message]) rescue {}
    igetui_opts = message

    user_ids = params[:user_ids] rescue []

    user_devices = UserDevice.where(user_id: user_ids,user_platform: params[:user_platform])
    client_id = message.dig('transmission_content', 'extras', 'extra', 'client_id')
    user_devices = user_devices.where.not(client_id: client_id) if client_id
    user_device_exist = user_devices.exists?

    unless user_device_exist
      return render json: { code: -1, message: 'user_device找不到,user_ids错误'}
    end

    # token_array = token_and_options(request)
    # if token_array.blank?
    #   return render json: { code: 401, message: 'token错误'}
    # end

    params[:sync_push] ||= false
    # if validate_token(params[:app_name], token_array.first)
      push_sync(params[:sync_push], {user_ids: user_ids, message: message, igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_list'), user_platform: params[:user_platform]})

      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    # else
    #   render json: { code: 401, message: 'token 错误/过期'}
    # end

  end

  def received
    param! :notification_id, Integer, required: true
    param! :user_id, Integer, required: true
    param! :app_type, String, required: true
    param! :device_model, String, required: true

    notification_stat = NotificationStatistic.find_by(user_id: params[:user_id], notification_id: params[:notification_id], app_type: params[:app_type])
    notification_stat.update(device_model: params[:device_model], is_received: true) if notification_stat.present?

    render json: { code: 0, message: "success" }
  end

  def push
    param! :app_name, String, required: true
    param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :sync_push, :boolean
    param! :igetui_opts, String
    param! :app_type, String

    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}
    device_ids_opts = JSON.parse(params[:device_ids_opts]) rescue {}

    # token_array = token_and_options(request)
    # if token_array.blank?
    #   return render json: { code: 401, message: 'token错误'}
    # end

    # if validate_token(params[:app_name], token_array.first)
      params[:sync_push] ||= false

      push_sync(params[:sync_push], {device_ids_opts: device_ids_opts, message: message, app_type: params[:app_type], igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_list')})
      # NotificationStatistic.create_by_params(igetui_opts, message, device_ids_opts, params[:app_type])


      render json: { code: 0, message: 'success', data: {remind: "请使用新的api,此api近期将不再维护，请尽快迁移到新的api,新api地址 http://apidoc.weiwenjia.com/docs/internal-dev-api/internal-dev-api-1boq6ctdm8m57"}}
     # else
     #  render json: { code: 401, message: 'token 错误/过期'}
     # end
  end

  private

  def push_sync(sync_push, opts={})
    if sync_push
      timeout_seconds = $redis.get('push_timeout').to_i
      if timeout_seconds > 0
        Timeout.timeout(timeout_seconds) {
          ::PushWorker.new.perform(opts)
        }
      else
        ::PushWorker.new.perform(opts)
      end
    else
      ::PushWorker.perform_async(opts)
    end
  end
end
